%SIMULAZIONE DEL POTENZIALE DI AZIONE 
% calcolare l'andamento del potenziale di membrana quando viene applicato
% un impulso di corrente di energia data 
%condizioni: assone gigante di calamaro T=6.3 C

clear variables 
close all
clc


%% costanti

GNaM=120; %[mS/cm2]
GKM=36; %[mS/cm2]
GL=0.3; %[mS/cm2]
CM=1; %[uF/cm2]
Vr=-65; %[mv]
VNa=50; %[mv]
VK=-77; %[mv]
VL=-54.387; %[mv]


%% condizioni iniziali : le calcolo come alpha(V=Vr)/(alpha(V=Vr)+beta(V=Vr))

%condizioni iniziale n_inf(V=Vr) h_inf(V=Vr)  m_inf(V=Vr) 
V0=Vr;
dv=0;

%utilizzo le formule ricavate da HH
an0=(0.01*(10-(dv)))/(exp((10-dv)/10)-1);
bn0=0.125/(exp(dv/80));
am0=(0.1*(25-(dv)))/(exp((25-dv)/10)-1); 
bm0=4/(exp(dv/18)); 
ah0=0.07/(exp(dv/20));
bh0=1/(exp((30-dv)/10)+1); 

n0=an0/(an0+bn0); %0.3177
m0=am0/(am0+bm0); %0.0529
h0=ah0/(ah0+bh0); %0.5961

% ind_r=find(Vm==V0);
% %le condizioni iniziali per n,m,h dipendono dalla tensione
% n0=n_inf(ind_r); %0.3177;
% m0=m_inf(ind_r); %0.0529;
% h0=h_inf(ind_r); %0.5961;
%% variabili di integrazione

%NB: i coefficienti di vel di transizione sono in 1/msec!

tmax=30; % [msec] Durata della simulazione
dt=0.01;% [ms] intervallo tra due passi della della simulazione 
steps=round(tmax/dt);
t=(0:steps-1)*dt;


l=20; %cm 
dx=0.1;
stepx=l/dx; 

%% inizializzo le variabili con la giusta dimensione 

  Vm = ones(steps,stepx)*V0;
  m = ones(steps,stepx)*m0;
  h = ones(steps,stepx)*h0;
  n = ones(steps,stepx)*n0;
  %GK=zeros(steps,stepx);
  %GNa=zeros(steps,stepx);
  %GTot=zeros(steps,stepx); %conduttanza totale
  %IK=zeros(steps,stepx);   %corrente potassio
  %INa=zeros(steps,stepx);  %corrente sodio 
  %IL=zeros(steps,stepx);   %corrente leakege
  %Ii=zeros(steps,stepx);   %corrente ionica 
  I=zeros(steps,1);    %corrente di membrana che impongo dall'esterno (ingresso forzante) 
  IM=zeros(steps,stepx);
  
 
%%  Corrente di stimolazione

%si fissa la "carica" dello stimolo, se poi fissiamo la durata otteniamo la
%corrente Is (carica integrale della corrente nel tempo) 

  stimolo=20e-9; %C/cm2 %soglia 6.50 
% stimolo=6e-9; %C/cm2
%stimolo=7e-9; %C/cm2
  durata=0.2; %msec
  IS=(stimolo)/(durata*1e-3); %corrente in A/cm2
  ncampioni=round(durata/dt); %numero di passi o campioni relativi alla durata dell'impulso
  %ii=(12e-9)/(2e-3);
  %ncampioni=2/dt;
  IS=IS*10^6;% [uA/cm2]
  I(5*ncampioni:5*ncampioni+ncampioni+ncampioni)=IS;
  figure
  plot(t,I);
  
%% refrattario (calcoli fatti per stimolo di 20e-9)
%  N=30; %8-37 assoluto;  N=60 o 80  realativo il solito stimolo di prima non basta, uno pi? alto si (4Is) (Is) non ce la fa nel relativo h deve essere sufficientemente alto, vedi figura finale
%  I(N*ncampioni:N*ncampioni+ncampioni)=4*IS; %REFRATTARIO: con  stimolo (Riperto lo stimolo mentre la membrana ? attivata)
  
  
  
%% firing periodico
  %I(1:end)=10; %per firing periodico -> piccola corrente costante
  
%%  setto le condizioni iniziali

  
%   Vm(1)=V0;
%   m(1)=m0;
%   n(1)=n0;
%   h(1)=h0;
  
%% Metodo Eulero: soluzione di Im=Ic+Ii -> impulso non propagante (space clamp) stimolato con una corrente esterna
%   



    
a=0.0238; %cm
R=34.5/1000; %kohm*cm
%figure(1)
%hold on
x=(0:stepx-1)*dx;

for i=1:steps-1 %tempo
for j=1:stepx-1 %Spazio





    
    %conduttanze
      GK=GKM*(n(i,j)^4);
      GNa=GNaM*(m(i,j)^3)*h(i,j);
      %GTot=GL+GK(i,j)+GNa(i,j);
      
      %correnti
      IK=GK*(Vm(i,j)-VK); %K
      INa=GNa*(Vm(i,j)-VNa); %Na
      IL=GL*(Vm(i,j)-VL); %L
      Ii=IK+INa+IL;
      
      if j==1
          II=1/dx*(I(i)/(2*pi*a)-(a/(R*dx))*(Vm(i,j)-Vm(i,j+1)));
      else
          II=(a/(2*R*dx^2))  * (Vm(i,j-1)-2*Vm(i,j)+Vm(i,j+1));     
      end
       IM(i,j)=II; 
      % III(i,j)=Ii;
      % IIIC(i,j)=II-Ii;
     
          
            
      Vm(i+1,j)=Vm(i,j)+dt*( II- Ii)/CM;
      
      
      dv=Vm(i,j)-Vr; % cambia Vm -> cambiano i coefficienti alpha e beta
      
      an=(0.01*(10-(dv)))/(exp((10-dv)/10)-1);
      bn=0.125/(exp(dv/80));
      am=(0.1*(25-(dv)))/(exp((25-dv)/10)-1); 
      bm=4/(exp(dv/18));
      ah=0.07/(exp(dv/20));
      bh=1/(exp((30-dv)/10)+1); 

      % rquazioni differenziali coefficienti alpha e beta
      n(i+1,j)= n(i,j)+dt*(  an*(1-n(i,j))-bn*n(i,j) );
      m(i+1,j)= m(i,j)+dt*( am*(1-m(i,j))-bm*m(i,j) );
      h(i+1,j)= h(i,j)+dt*( ah*(1-h(i,j))-bh*h(i,j) );
      
   
          
    
end
%    if mod(i,500)==0
%        sss=Vm(i,:);
%           plot(x,sss);
%     end
end

%% hold on
% for i=30:50:stepx-1
%     subplot(2,1,1)
%     plot(t,Vm(:,i),'b')
%     hold on
%     subplot(2,1,2)
%     hold on
%     plot(t,IM(:,i),'b')
% end

figure
% hold on
s1=501;
plot(x,Vm(s1,:));
hold on
s2=1001;
plot(x,Vm(s2,:));
s3=1501;
plot(x,Vm(s3,:));
legend(    num2str(t(s1))   ,  num2str(t(s2)) , num2str(t(s3))   );

figure
subplot(2,1,1)
s1=501;
plot(x,Vm(s1,:));
hold on
s2=1001;
plot(x,Vm(s2,:));
s3=1501;
plot(x,Vm(s3,:));
legend(    num2str(t(s1))   ,  num2str(t(s2)) , num2str(t(s3))   );
subplot(2,1,2)
s1=501;
plot(x,IM(s1,:));
hold on
s2=1001;
plot(x,IM(s2,:));
s3=1501;
plot(x,IM(s3,:));
%plot(x,IM(s1,:));
%plot(x,IM(s2,:));
%plot(x(1:end-1),III(s3,:));
legend(    num2str(t(s1))   ,  num2str(t(s2)) , num2str(t(s3))   );

%III(i,j)=Ii;
%       IIIC(i,j)=II-Ii;;
% for i=1:500:steps-2
%     subplot(2,1,1)
%     plot(x,Vm(i,:))
%     hold on
%     subplot(2,1,2)
%     hold on
%     plot(x,IM(i,:))
% end
%   




 
% %% Grafico la corrente di stimolo e il potenziale di azione
% 
% figure
% plot(t,I);
% xlabel('t [msec]');
% legend('IS');
% ylabel('[\mu A/cm^2]'); %mS*mV
% 
% figure
% plot(t,Vm);
% ylabel('[mV]');
% xlabel('t [msec]');
% hold on
% plot(t,ones(size(Vm))*Vr,'-.');
% legend('Vm','Vr');
% ylim([-90 40]);
% 
% 
% %% Grafico  conduttanze
% 
% figure
% subplot(2,1,1)
% plot(t,GK);
% hold on
% plot(t,GNa);
% plot(t,GTot,'g');
% scaleV=40/100;
% plot(t,(Vm-Vr)*scaleV,'k--');
% legend('GK','GNa','Gtot','\Delta Vm');
% xlabel('t [msec]');
% ylabel('mS/cm^2');
% subplot(2,1,2)
% plot(t,h)
% xlabel('t [msec]');
% legend('h');
% 
% 
% 
% %% refrattario
% 
% %  figure
% %  subplot(2,1,1)
% %  plot(t,h)
% %  legend('h')
% %  xlabel('t [msec]');
% %  subplot(2,1,2)
% %  plot(t,I)
%  %xlabel('t [msec]');